﻿module app {

    export interface IPerson {
        lastName: string;
        firstName: string;
    }

    export interface IPersonScope extends ng.IScope {
        person: IPerson;
        populate(): void;
    }

    export class MainController {
        constructor($scope: IPersonScope, personService: app.services.IPersonService) {
            $scope.person = {
                lastName: "",
                firstName: ""
            };
            $scope.populate = () => {
                personService.getPerson().then(
                    (callback: ng.IHttpPromiseCallbackArg<app.IPerson>) => {
                        $scope.person = callback.data;
                    },
                    (response: ng.IHttpPromiseCallbackArg<app.IPerson>) => {
                        alert(response.statusText);
                    });
            };
        }
    }

    export class MakeRedDirective implements ng.IDirective {
        link(scope: ng.IScope,
            instanceElement: ng.IAugmentedJQuery,
            instanceAttributes: ng.IAttributes,
            controller?: any,
            transclude?: ng.ITranscludeFunction
            ): void {

            var clickHanlder: () => void;
            clickHanlder = () => {
                instanceElement.addClass("red");
            };
            instanceElement.click(clickHanlder);
        }
    }

    angular.module("app", ["app.services"])
        .controller("mainController", ["$scope", "personService", ($scope: IPersonScope, personService: app.services.IPersonService) => {
            return new MainController($scope, personService);
        }])
        .directive("makeRed", [() => {
            return new MakeRedDirective();
        }]);
}

module app.services {
    export interface IPersonService {
        getPerson: () => ng.IHttpPromise<app.IPerson>;

    };

    export class PersonService implements IPersonService {
        constructor(private $http: ng.IHttpService) {

        }
        getPerson(): ng.IHttpPromise<app.IPerson> {
            return this.$http.get("data.txt");
        }
    }
    angular.module("app.services", [])
        .factory("personService", ["$http", ($http: ng.IHttpService) => {
            return new PersonService($http);
        }]);
}
